package test.active;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

// the display of the diary
class DiaryDisplay extends Frame {

  private String dayNames[] = {"Monday", "Tuesday", "Wednesday", "Thursday",
			       "Friday", "Saturday", "Sunday"};
  private Button buttons[];   
  private TextField entries[];
  private Label dayName;
  private Button lastPressedButton;
  private int focusedField;
  private int currentDay;
  private int focusedTime;
  private ButtonActionListener buttonActionListener;
  private EntryActionListener entryActionListener;
  private EntryFocusListener entryFocusListener;
  private DataInputStream in;
  private DataOutputStream out;

  class DiaryButtons extends Panel {

    DiaryButtons() {
	    
      String names[] = {"Mon","Tue","Wed","Thu","Fri","Sat","Sun","Exit"};
      buttons = new Button[names.length];
	   
      for (int i=0; i<buttons.length; i++) {
	buttons[i] = new Button(names[i]);
	buttons[i].addActionListener(buttonActionListener);
	add(buttons[i]);
      }
	    
      lastPressedButton = buttons[0];
    }
  }
	
  class DayDisplay extends Panel {

    DayDisplay() {

      String times[] = {"08.00-10.00","10.00-12.00","12.00-14.00",
			"14.00-16.00","16.00-18.00"};
      entries = new TextField[times.length];

      dayName = new Label(dayNames[0]);
      TimesDisplay timesDisplay = new TimesDisplay(times);
      EntriesDisplay entriesDisplay = new EntriesDisplay(entries);

      setLayout(new BorderLayout());

      add(dayName, BorderLayout.NORTH);
      add(timesDisplay, BorderLayout.WEST);
      add(entriesDisplay, BorderLayout.EAST);	    
    }
  }
	    
  class TimesDisplay extends Panel {

    TimesDisplay(String times[]) {
	    
      setLayout(new GridLayout(times.length,1));
		      
      for (int i=0; i<times.length; i++) {
	add(new Label(times[i]));
      }
    }
  }

  class EntriesDisplay extends Panel {
    
    EntriesDisplay(TextField entries[]) {

      setLayout(new GridLayout(entries.length,1));
	    
      for (int i=0; i<entries.length; i++) {
	entries[i] = new TextField(35);
	entries[i].addActionListener(entryActionListener);
	entries[i].addFocusListener(entryFocusListener);
	add(entries[i]);
      }
    }
  }

  class ButtonActionListener implements ActionListener {

    public void actionPerformed(ActionEvent e) {
      if (e.getSource() == buttons[7]) {
	try {
	  out.writeInt(DiaryDisplayInterface.EXIT);
	  out.flush();
	} catch(Throwable th) {
	  System.err.println("DiaryDisplay: ButtonActionListener: "+th.getClass().getName()+
			     " "+th.getMessage());
	}				
	System.exit(0);
      } else {
	lastPressedButton=(Button)e.getSource();
	for (int i=0; i<entries.length;i++) {
	  entries[i].setText("");
	}
	for(int i=0; i<buttons.length; i++) {
	  if (buttons[i]==e.getSource()) {
	    dayName.setText(dayNames[i]);
	    currentDay=i;
	    break;
	  }
	}
	  
	try {
	  out.writeInt(DiaryDisplayInterface.NEWDAY);
	  out.writeInt(currentDay);
	  out.flush();
	} catch(Throwable th) {
	  System.err.println("DiaryDisplay: ButtonActionListener: "+th.getClass().getName()+
			     " "+th.getMessage());
	}				
      }
    }
  }

  class EntryActionListener implements ActionListener {

    public void actionPerformed(ActionEvent e) {
      for (int i=0; i<entries.length; i++) {
	if (e.getSource() == entries[i]) {
	  lastPressedButton.requestFocus();
	  break;
	}
      }			
    }
  }

  class EntryFocusListener implements FocusListener {

    public void focusGained (FocusEvent e) {
      for (int i=0; i<entries.length; i++) {
	if (e.getSource() == entries[i]) {		   
	  focusedField = i;
	  break;
	}
      }
    }

    public void focusLost (FocusEvent e) {
      for (int i=0; i<entries.length; i++) {
	if (e.getSource() == entries[i]) {
	  try {
	    out.writeInt(DiaryDisplayInterface.NEWENTRY);
	    out.writeInt(currentDay);				
	    out.writeInt(i);				
	    out.writeUTF(entries[i].getText());
	    out.flush();
	  } catch(Throwable th) {
	    System.err.println("DiaryDisplay: EntryActionListener: "+
			       th.getClass().getName()+" "+th.getMessage());
	  }
			
	  focusedField = -1;
	  break;
	}
      }
    }
  }

  private class DisplayThread extends Thread {

    DisplayThread() 
    {
      super();
      this.start();
    }
	
    public void run() 
    {
      try {
	for (;;) {
	  int day=in.readInt();
	  int time=in.readInt();
	  String entry=in.readUTF();
	  if (currentDay==day && focusedField!=time) {
	    entries[time].setText(entry);
	  }
	}
      } catch(Throwable th) {
	System.err.println("DiaryDisplay: DisplayThread: "+th.getClass().getName()+" "+
			   th.getMessage());
      }	
    }
  }      

  DiaryDisplay(String title) {
    super(title);

    focusedField = -1;

    in = new DataInputStream(System.in);
    out = new DataOutputStream(System.out);

    buttonActionListener = new ButtonActionListener();
    entryActionListener = new EntryActionListener();
    entryFocusListener = new EntryFocusListener();

    setLayout(new BorderLayout());		
    add(new DiaryButtons(), BorderLayout.NORTH);
    add(new DayDisplay(), BorderLayout.CENTER);

    try {
      out.writeInt(DiaryDisplayInterface.NEWDAY);
      out.writeInt(currentDay);
      out.flush();
    } catch(Throwable th) {
      System.err.println("DiaryDisplay: Constructor: "+th.getClass().getName()+" "+
			 th.getMessage());
    }

    pack();
    show();

    new DisplayThread();
  }

  public static void main(String argv[]) {
	
    DiaryDisplay display = new DiaryDisplay("Diary");
  }
}
