package test.active;

import java.io.*;
import java.net.*;
import replica.*;
import meta.*;

// interface to connect a diary and its display
class DiaryDisplayInterface {

  Diary diary;
  Process proc;
  DataInputStream in;
  DataOutputStream out;
  InputStream err;
  ErrorThread errThread;
   
  static final int NEWDAY   = 0;
  static final int NEWENTRY = 1;
  static final int EXIT     = 2;
    
  class DiaryThread extends Thread {
	
    DiaryDisplayInterface diaryInterface;
	
    DiaryThread() 
    {
      super();
      this.start();
    }
	
    public void run() 
    {
      try {
	for (;;) {
	  int mess = in.readInt();
	  if (mess == DiaryDisplayInterface.NEWDAY) {
	    int day=in.readInt();
	    for(int i=0; i<5; i++) {
	      notifyDisplay(day,i,diary.getEntryR(day,i));
	    }
	  } else if (mess == DiaryDisplayInterface.NEWENTRY) {
	    int day=in.readInt();
	    int time=in.readInt();
	    String entry=in.readUTF();
	    diary.setEntry(day,time,entry);
	  } else {
	    break;
	  }	
	}      
      } catch(Throwable th) {
	System.out.println("DiaryDisplayInterface: DiaryThread: "+
			   th.getClass().getName()+" "+th.getMessage());
      }
    }
  }
  
  class ErrorThread extends Thread {
	
    DiaryDisplayInterface diaryInterface;
	
    ErrorThread() 
    {
      super();
      this.start();
    }
	
    public void run() 
    {
      try {
	int b;
	while((b=err.read()) != -1) {
	  System.err.write(b);
	}
      } catch(Throwable th) {
	System.out.println("DiaryDisplayInterface: ErrorThread: "+th.getClass()
			   .getName()+" "+th.getMessage());
      }	
    }
  }     

  DiaryDisplayInterface(Diary newDiary) {
	
    diary = newDiary;

    try {

      proc = Runtime.getRuntime().exec(new String[] {"test/active/StartDiaryDisplay"});

      in = new DataInputStream(proc.getInputStream());
      out = new DataOutputStream(proc.getOutputStream());
      err = proc.getErrorStream();
    } catch(Throwable th) {
      System.out.println("DiaryDisplayInterface: constructor: "+th.getClass().
			 getName()+" "+th.getMessage());
    }

    new DiaryThread();
    errThread = new ErrorThread();

    new AccessControlObject(this, new AccessTransmitSystemObject());
  }
    
  synchronized  void notifyDisplay(int day, int time, String entry) {
	
    if (entry != null) {
      try {
	out.writeInt(day);
	out.writeInt(time);
	out.writeUTF(entry);
	out.flush();
      } catch(Throwable th) {
	System.out.println("DiaryDisplayInterface: notifyDisplay: "+
			   th.getClass().getName()+" "+th.getMessage());
      }
    }
  }
}
	
