package test.passive;

import replica.*;
import replica.exception.*;
import java.util.*;
import java.net.*;

/**
 * This class is an executable class. It demonstrates how the passive replication works.
 *
 * @author Markus Jger
 * @version 1.0
 * 
 * @see Writer
 */
public class Reader {
  public static void main(String argv[]) {    
    
    // get port or 0
    int contactPort = 0;    
    if (argv.length >=1) {
      try {
	contactPort = Integer.parseInt(argv[0]);
      } catch (NumberFormatException e) {
	contactPort = 0;
      }
    }
    
    // start ReplicationEnvironment
    new ReplicationEnvironment();
    
    // the replicated object
    OrderObject order=null;
    
    // no port --> create new object
    if (contactPort == 0) {
      // create an object 
      order = new OrderObject();
      new AccessControlObject(order, new PassiveReplication());
    } else {
      try {
	// get remote NameService
	NameService remoteNS = NameService.getRemoteNS(contactPort);
	// get replica
	order = (OrderObject)remoteNS.get("OrderObject");
      } catch(Exception ex) {
	System.out.println("Exception(Reader): "+ex.getClass().getName()+" "+ex.getMessage());
	System.exit(1);
      }
    }

    // register at my own NameService
    NameService.getLocalNS().register("OrderObject", order);

    // generate delay time
    Random ran = new Random();
    int r = ran.nextInt();
    if (r<0) r= -r;
    int t=r/2000;
    r=r-(t*2000)+1000;
  
    // read from OrderObject
    while (true) {
      try {
	System.out.println("The next number will be "+order.getNumber()+" ("+
			   RemoteObjectControl.ownPort+")");
      } catch (ReplicaException ex) {
	System.out.println("Exception(Reader): "+ex.getClass().getName()+" "+ex.getMessage());
	System.exit(1);
      }

      try {
	Thread.sleep(r);
      } catch (InterruptedException ex) {}
    }
  }
}

