package test.remote;

import replica.*;
import java.net.*;

/**
 * This class is an executable class. It demonstrates how the remote access works.
 *
 * @author Markus Jger
 * @version 1.0
 * 
 * @see Server 
 * @see Update
 */
public class Query {
  public static void main(String argv[]) {    
    
    if (argv.length <1) {
      System.out.println("Usage: Query <key> [port]");
      System.exit(1);
    }

    // determin port
    int contactPort = 12345;
    if (argv.length >1) { 
      try {
	contactPort = Integer.parseInt(argv[1]);
      } catch (NumberFormatException e) {
	contactPort = 12345;
      }
    }

    // start ReplicationEnvironment
    new ReplicationEnvironment();
    
    try {
      // get remote NameService
      NameService remoteNS = NameService.getRemoteNS(InetAddress.getLocalHost().
						     getHostName(),contactPort);
	
      // get database
      Database base = (Database)remoteNS.get("Database");	  
	
      // query database
      System.out.println("The value is: "+base.retrieve(argv[0]));
   
    } catch(Exception ex) {
	System.out.println("Exception(Query): "+ex.getClass().getName()+" "+ex.getMessage());
    }

    // exit
    System.exit(0);
  }
}
