package test.remote;

import replica.*;
import java.net.*;

/**
 * This class is an executable class. It demonstrates how the remote access works.
 *
 * @author Markus Jger
 * @version 1.0
 * 
 * @see Server 
 * @see Query
 */
public class Update {
  public static void main(String argv[]) {    
    
    if (argv.length <2) {
      System.out.println("Usage: Update <key> <value> [port]");
      System.exit(1);
    }

    // determin port
    int contactPort = 12345;
    if (argv.length >2) { 
      try {
	contactPort = Integer.parseInt(argv[2]);
      } catch (NumberFormatException e) {
	contactPort = 12345;
      }
    }
    
    // start ReplicationEnvironment
    new ReplicationEnvironment();

    try {
      // get remote NameService
      NameService remoteNS = NameService.getRemoteNS(InetAddress.getLocalHost().
						     getHostName(),contactPort);

      // get database
      Database base = (Database)remoteNS.get("Database");	  
	
      // update database
      base.put(argv[0], argv[1]);
	
    } catch(Exception ex) {
      System.out.println("Exception(Update): "+ex.getClass().getName()+" "+ex.getMessage());
    }

    // exit
    System.exit(0);
  }
}

